/*
 * Example1.java
 *
 * Umgestellt auf lokales LgDatum statt java.sql.Date
 * Created on 28. Februar 2001, 15:50
 */

package bank3tier._lgTest;

import java.util.List;

import bank3tier.lg.LgDatum;
import bank3tier.lg.LgKonto;
import bank3tier.lg.LgKunde;
import bank3tier.lg.LgSitzung;

/**
  Zweck: Erzeugt einige Kunden sowie einige von diesen verwaltete Konten
  und führt einige Überweisungen aus.

  @author  Christoph Knabe
 */
public class ManualTransactionExample {

    private final LgSitzung _lgSitzung;

    /**Do some work using the application logic layer (Lg) */
    public static void main (final String[] i_args){
        try{
            final ManualTransactionExample example = new ManualTransactionExample(new LgSitzung());
        	example.work();
        }catch(final Exception ex){
        	multex.Msg.printReport(ex);
        	System.exit(1);
        }
    }

    /**create application object and and empty database, then do modifications.*/
    public ManualTransactionExample(final LgSitzung lgSitzung) throws Exception {
        _lgSitzung = lgSitzung;
    }//work

    /**Does the example modifications to the database as one big manual transaction, either completely or nothing at all.*/
	private final void work() throws Exception {
        _lgSitzung.beginTransaction();
        final boolean ok = false;
		try {
			//Einige Kunden erfassen:
			final LgKunde mueller
            = _lgSitzung.kundeErfassen("Manfred Müller", LgDatum.fromString("02.06.1967"));

            final LgKunde gans
            = _lgSitzung.kundeErfassen("Gustav Gans", LgDatum.fromString("17.06.1953"));

            final LgKunde sorglos
            = _lgSitzung.kundeErfassen("Susi Sorglos", LgDatum.fromString("09.11.1989"));

			//Für diese Kunden Konten eröffnen:
            final LgKonto muellerGiroKonto = mueller.kontoEroeffnen("muellerGiroKonto").getKonto();
            final LgKonto muellerSparKonto = mueller.kontoEroeffnen("muellerSparKonto").getKonto();
            final LgKonto gansKonto = gans.kontoEroeffnen("gansKonto").getKonto();
            final LgKonto guthsmuthsVereinskonto = gans.kontoEroeffnen("guthsmuthsVereinskonto").getKonto();

            //Skoric als Verwalter für gansKonto einsetzen:
            gans.verwalterHinzufuegen(gansKonto, sorglos);
            //Müller als Verwalter für Konto des TSV Guthsmuths einsetzen:
            gans.verwalterHinzufuegen(guthsmuthsVereinskonto, mueller);

			//Einige Überweisungen tätigen:
            muellerGiroKonto.ueberweisen(muellerSparKonto, 200000, mueller);
            muellerGiroKonto.ueberweisen(guthsmuthsVereinskonto, 2000, mueller);
            gansKonto.ueberweisen(muellerGiroKonto, 1998, gans);

			//Zeige Liste aller Kunden:
            final List<LgKunde> kundeList = this._lgSitzung.alleKundenSuchen();
			System.out.println("Liste aller Kunden");
			System.out.println("==================");
			System.out.println("oid,\tName,\tGeburt");
			for(final LgKunde k: kundeList){
			  System.out.println(k.getOid() + ",\t" + k.getName() + ",\t" + k.getGeburt());
			}
			//Zeige Status und Kontostand aller von Kunde 0 verwalteten Konten:
			final LgKunde kunde0 = kundeList.get(0);
			kunde0.printKontenStaende();

			//Demonstriert eindeutiges Suchen:
			final LgDatum suchDatum = LgDatum.fromString("17.06.1953");
			final LgKunde geb530617Kunde = _lgSitzung.kundenSuchen(suchDatum);
            System.out.println("Zeige den Kunden, der am " + suchDatum + " geboren wurde:");
			System.out.println(geb530617Kunde);
		}finally{
			_lgSitzung.endTransaction(ok);
		}
	}

}//ManualTransactionExample