/*
 * SomeTransactionsExample.java
 *
 * Umgestellt auf lokales LgDatum statt java.sql.Date
 * Created on 28. Februar 2001, 15:50
 */

package bank3tier._lgTest;

import java.util.List;

import bank3tier.lg.LgDatum;
import bank3tier.lg.LgKonto;
import bank3tier.lg.LgKunde;
import bank3tier.lg.LgSitzung;
import bank3tier.lg.LgTransaction;

/**
  Zweck: Erzeugt einige Kunden sowie einige von diesen verwaltete Konten
  und führt einige Überweisungen aus.

 * @author Christoph Knabe, Beuth University of Applied Sciences, Berlin, Germany
 * @since 2014-07-14
 */
public class SomeTransactionsExample {

    private final LgSitzung _lgSitzung;

    /**Do some work using the application logic layer (Lg) */
    public static void main (final String[] i_args){
        try{
            final SomeTransactionsExample example = new SomeTransactionsExample(new LgSitzung());
        	example.work();
        }catch(final Exception ex){
        	multex.Msg.printReport(ex);
        	System.exit(1);
        }
    }

    /**create application object and and empty database, then do modifications.*/
    public SomeTransactionsExample(final LgSitzung lgSitzung) throws Exception {
        _lgSitzung = lgSitzung;
    }

    /** Does the example modifications to the database as some transactions. */
	private final void work() throws Exception {
		/** Hilfsklasse zur Aufnahme der Transaktionsergebnisse. */
		class Mueller {
			final LgKunde kunde;
			final LgKonto giroKonto, sparKonto;
			Mueller(final LgKunde kunde, final LgKonto giroKonto, final LgKonto sparKonto){
				this.kunde = kunde;   this.giroKonto = giroKonto;  this.sparKonto = sparKonto;
			}
		}
		final Mueller mueller = new LgTransaction<Mueller>(_lgSitzung) {
			/** Kunden Müller und seine Konten als eine Transaktion erfassen. */
			@Override
			protected Mueller execute() throws Exception {
				final LgKunde muellerKunde = _lgSitzung.kundeErfassen("Manfred Müller", LgDatum.fromString("02.06.1967"));
	            final LgKonto giroKonto = muellerKunde.kontoEroeffnen("muellerGiroKonto").getKonto();
	            final LgKonto sparKonto = muellerKunde.kontoEroeffnen("muellerSparKonto").getKonto();
				return new Mueller(muellerKunde, giroKonto, sparKonto);
			}
		}.getResult();
        final LgKunde gans = new LgTransaction<LgKunde>(_lgSitzung) {
			@Override
			protected LgKunde execute() throws Exception {
				return _lgSitzung.kundeErfassen("Gustav Gans", LgDatum.fromString("17.06.1953"));
			}
		}.getResult();
		final LgKunde sorglos = new LgTransaction<LgKunde>(_lgSitzung) {
			@Override
			protected LgKunde execute() throws Exception {
	            return _lgSitzung.kundeErfassen("Susi Sorglos", LgDatum.fromString("09.11.1989"));
			}
		}.getResult();
        final LgKonto gansKonto = new LgTransaction<LgKonto>(_lgSitzung) {
			@Override
			protected LgKonto execute() throws Exception {
				final LgKonto result = gans.kontoEroeffnen("gansKonto").getKonto();
	            //Sorglos als zusätzliche Verwalterin für gansKonto einsetzen:
	            gans.verwalterHinzufuegen(result, sorglos);
	            return result;
			}
		}.getResult();
		final LgKonto guthsmuthsVereinskonto = new LgTransaction<LgKonto>(_lgSitzung) {
			@Override
			protected LgKonto execute() throws Exception {
	            final LgKonto result = gans.kontoEroeffnen("guthsmuthsVereinskonto").getKonto();
	            //Müller als Verwalter für Konto des TSV Guthsmuths einsetzen:
	            gans.verwalterHinzufuegen(result, mueller.kunde);
	            return result;
			}
		}.getResult();
		//Bei dieser Transaktion wird kein Ergebnis benötigt, daher Ergebnistyp Void:
		new LgTransaction<Void>(_lgSitzung) {
			@Override
			protected Void execute() throws Exception {
				//Einige Überweisungen tätigen:
	            mueller.giroKonto.ueberweisen(mueller.sparKonto, 200000, mueller.kunde);
	            mueller.giroKonto.ueberweisen(guthsmuthsVereinskonto, 2000, mueller.kunde);
	            gansKonto.ueberweisen(mueller.giroKonto, 1998, gans);
	            return null;
			}
		};
		//Bei dieser Transaktion wird kein Ergebnis benötigt, daher Ergebnistyp Void:
		new LgTransaction<Void>(_lgSitzung) {
			@Override
			protected Void execute() throws Exception {
				//Zeige Liste aller Kunden:
	            final List<LgKunde> kundeList = _lgSitzung.alleKundenSuchen();
				System.out.println("Liste aller Kunden");
				System.out.println("==================");
				System.out.println("oid,\tName,\tGeburt");
				for(final LgKunde k: kundeList){
				  System.out.println(k.getOid() + ",\t" + k.getName() + ",\t" + k.getGeburt());
				}
				//Zeige Status und Kontostand aller von Kunde 0 verwalteten Konten:
				final LgKunde kunde0 = kundeList.get(0);
				kunde0.printKontenStaende();

				//Demonstriert eindeutiges Suchen:
				final LgDatum suchDatum = LgDatum.fromString("17.06.1953");
				final LgKunde geb530617Kunde = _lgSitzung.kundenSuchen(suchDatum);
	            System.out.println("Zeige den Kunden, der am " + suchDatum + " geboren wurde:");
				System.out.println(geb530617Kunde);

				return null;
			}
		};
	}

}//SomeTransactionsExample