/*
 * LgKonto.java
 *
 * 2001-04-22  Knabe  extends DbObject
 * Created on 28. Februar 2001, 15:44
 */

package bank3tier.lg;
import javax.persistence.Entity;
import javax.persistence.Table;

import  bank3tier.db.DbObject;
import bank3tier.db.Pool;
import bank3tier.db.Pool.NoPersistentClassExc;
import static multex.MultexUtil.create;

/** * Ein Konto bei einer Bank.
 * @author  Christoph Knabe
 */
@Entity @Table(name="Konto")
public class LgKonto extends DbObject {
    //Attribute:
    private int saldo = 0;
    private java.util.Date eroeffnet = new java.util.Date();
    private String bezeichnung = null;

    /*package*/ LgKonto(){ /*Only necessary for Hibernate.*/ }
    
    /*package*/ LgKonto(final String bezeichnung){this.bezeichnung = bezeichnung;}
    /*package*/ LgKonto(final Pool io_pool){super(io_pool);}
    
    /**Liefert den aktuellen Kontosaldo in Cent.*/
    public int getSaldo(){return this.saldo;}

    /**Liefert die Bezeichnung des Kontos oder null.*/
    public String getBezeichnung() {
        return this.bezeichnung;
    }
    
    /**Der Mindestkontosaldo, der durch Verfügungen über das Konto nicht unterschritten werden darf.*/
    public static final int mindestSaldo = -1000000;

    /**Überweist den Geldbetrag i_betrag in Cent von diesem Konto auf das
     * Konto io_zielKonto.
    */
    public void ueberweisen(
        final LgKonto io_zielKonto, final int i_betrag,
        final LgKunde i_auftraggeber
    ) throws BetragExc, OhneRechteExc, MindestSaldoExc
    {
        if(i_betrag<=0){
            throw create(BetragExc.class, i_betrag);
        }
        try {
            final Pool pool = getPool();
            
            //0. Reattachen der beiden beteiligten Konten:
            //TODO Signatur umbauen oder Reattachen der beiden Konten in Aspekt auslagern! 07-08-27 Knabe
            pool.reattach(this);
            pool.reattach(io_zielKonto);
            
            //1. Fehlerprüfung:
            /*if(!pool.areLinked(i_auftraggeber, LgKontozugriff.class, this))
            {throw new OhneRechteExc(i_auftraggeber, this);}*/
            OhneRechteExc.check(pool, i_auftraggeber, this);
            final int neuSaldo = this.saldo - i_betrag;
            if(neuSaldo < mindestSaldo){throw create(MindestSaldoExc.class, neuSaldo, mindestSaldo);}

            //2. Veränderungen im Arbeitsspeicher vornehmen:
            this.saldo = neuSaldo;
            io_zielKonto.saldo += i_betrag;
            
            //3. Veränderungen persistieren:
        }catch(OhneRechteExc ex){throw ex;
        }catch(MindestSaldoExc ex){throw ex;
        }catch(final Exception ex){throw new multex.Failure(
            "Betrag {0} Cent konnte nicht überwiesen werden", ex, new Long(i_betrag)
            );
        }
    }

    @Override
    public String toString(){
        final String localClassName = getClass().getSimpleName();
        return localClassName + " {oid=" + getOid() + ", saldo=" + this.saldo + ", eroeffnet=" + this.eroeffnet + ", bezeichnung=" + this.bezeichnung + "}";
    }
    
    /**Kunde mit OID {0} ist kein berechtigter Verwalter des Kontos mit OID {1}*/
    public static class OhneRechteExc extends multex.Exc {
         static void check(final Pool pool, final LgKunde i_auftraggeber, final LgKonto i_konto) throws NoPersistentClassExc, OhneRechteExc {
            if(!pool.areLinked(i_auftraggeber, LgKontozugriff.class, i_konto)){
            	throw create(OhneRechteExc.class, i_auftraggeber, i_konto);
            }
        }
    }

    /**Unzulässiger Überweisungsbetrag {0} Cent. Muss größer als 0 sein!*/
    public static class BetragExc extends multex.Exc {}
    
    /**Neuer Saldo {0} Cent würde Mindestsaldo {1} Cent unterschreiten*/
    public static class MindestSaldoExc extends multex.Exc {}

    /**Kunde mit OID {0} ist schon Verwalter von Konto mit OID {1}*/
    public static class SchonVerwalterExc extends multex.Exc {}

}

