/*
 * LgKunde.java
 *
 * 2001-04-22  Knabe  extends DbObject
 * Created on 28. Februar 2001, 15:34
 */

package bank3tier.lg;
import java.util.List;

import javax.persistence.Basic;
import javax.persistence.Entity;
import javax.persistence.Table;
import javax.persistence.Transient;

import bank3tier.db.DbObject;
import bank3tier.db.Pool;
import bank3tier.lg.LgKonto.OhneRechteExc;
import bank3tier.ut.StringUtil;
import bank3tier.ut.StringUtil.EmptyFieldExc;

import static multex.MultexUtil.create;

/**
 * Ein Kunde einer Bank.
 * @author  Christoph Knabe
 */
@Entity @Table(name="Kunde")
public class LgKunde extends DbObject {
    
    
    private String name;
    
    private java.sql.Date geburt;
    public static final String geburtFIELD = "geburt";
    
    public void setName(final String i_name){this.name = i_name;}
    public String getName(){return this.name;}

    public void setGeburt(final LgDatum i_geburt){this.geburt = i_geburt.toDate();}
    @Transient
    public LgDatum getGeburt(){return new LgDatum(this.geburt);}
    
    /*public List<LgKontozugriff> getVerwaltet(){
        return new ArrayList<LgKontozugriff>(this.verwaltet);
    }*/

    /**Ändert das Kundenobjekt als externe, geprüfte Operation*/
    public void aendern(){
        try {
            getPool().reattach(this);
        }catch(Exception ex){throw new multex.Failure(
            "Es war nicht möglich, den Kunden {0} zu ändern", ex, this.name);
        }
    }

    /**Löscht das Kundenobjekt als externe geprüfte Operation.*/
    public void loeschen()throws KundeHatNochKontenExc{
        try {
            final Pool pool = getPool();
            if(pool.hasLinks(LgKontozugriff.class, this)){
                throw create(KundeHatNochKontenExc.class, getName());
            }
            pool.delete(this);
        }catch(KundeHatNochKontenExc ex){throw ex;
        }catch(Exception ex){throw new multex.Failure(
            "Es war nicht möglich, den Kunden {0} zu löschen", ex, getName());
        }
    }

    /**Setzt diese Attribute im aktuellen Objekt.
      Als interne, unpersistente Operation.
    */
    public void setzen(final String i_name, final LgDatum i_geburt){
		System.out.println(this.getClass().getSimpleName() + ".setzen(name="+ i_name + ", geburt=" + i_geburt + ")");
    	StringUtil.EmptyFieldExc.check(i_name, "name", this);
        name = i_name;   geburt = i_geburt.toDate();
    }

    /**Eröffnet ein neues Konto, dessen Inhaber der aktuelle Kunde wird.
      Als externe geprüfte Operation.
      @return das für Kunden und Konto eindeutige Verwaltet-Objekt
    */
    public LgKontozugriff kontoEroeffnen(final String bezeichnung){
        try {
            final LgKonto konto = new LgKonto(bezeichnung);
            getPool().insert(konto);
            final LgKontozugriff verwaltet = new LgKontozugriff(this, true, konto); //isInhaber
            getPool().insert(verwaltet);
            return verwaltet;
        }catch(Exception ex){throw new multex.Failure(
            "Es war nicht möglich, ein Konto zu eröffnen", ex);
        }
    }
    
    /**Trägt Kunden als weiteren Verwalter eines Kontos ein.
     * @param i_konto Das Konto, das einen weiteren Verwalter erhält.
     * @param i_kunde Der Kunde, der als weiterer Verwalter eingetragen wird.
     * @throws bank3tier.lg.LgKunde.SchonVerwalterExc i_kunde ist schon Verwalter des Kontos i_konto
     * @throws OhneRechteExc 
     */

    public LgKontozugriff verwalterHinzufuegen(final LgKonto i_konto, final LgKunde i_kunde) throws SchonVerwalterExc, OhneRechteExc {
        final Pool pool = getPool();
        try {
            LgKonto.OhneRechteExc.check(pool, this, i_konto);
            if(pool.areLinked(i_kunde, LgKontozugriff.class, i_konto)){
                throw create(SchonVerwalterExc.class, i_kunde, i_konto);
            }
            final LgKontozugriff verwaltet = new LgKontozugriff(i_kunde, false, i_konto); //kein Inhaber
            pool.insert(verwaltet);
            return verwaltet;
        }catch(final SchonVerwalterExc ex){throw ex;
        }catch(final OhneRechteExc ex){throw ex;
        }catch(final Exception ex){
            throw new multex.Failure(
                "Verwalter konnte nicht hinzugefügt werden.", ex
            );
        }
    }

    /**Listet alle Konten des aktuellen Kunden auf mit ihrem Saldo*/
    public void printKontenStaende(){
        final Pool pool = getPool();
        try{
            //Alle LgKontozugriff-Links dieses Kunden:
            final List<LgKontozugriff> verwaltetList = pool.getLinks(LgKontozugriff.class, this);
            System.out.println("-----------------------------");
            System.out.println("KontenStände von Kunde " + this.name + " #" + getOid());
            System.out.println("-----------------------------");
            System.out.println("KontoOid,\tStatus,\tSaldo");
            for(final LgKontozugriff verwaltet: verwaltetList){
                final String status = verwaltet.isInhaber() ? "Inhaber" : "Verwalter";
                final LgKonto konto = verwaltet.getKonto();
                System.out.println("#" + konto.getOid() + " " + status+ " " + konto.getSaldo());
            }
        }catch(Exception ex){throw new multex.Failure(
            "Fehler beim Ausgeben der KontenStände for Kunde OID {0}", ex, getOid());
        }
    }

    @Override
    public String toString(){
        final String localClassName = getClass().getSimpleName();
        return localClassName + " {oid=" + getOid() + ", name=" + this.name + ", geburt=" + this.geburt + "}";
    }

    /**Kunde mit OID {0} ist schon Verwalter von Konto mit OID {1}.*/
    public static class SchonVerwalterExc extends multex.Exc {
        private static final long serialVersionUID = -7763140002896436518L;
    }
    
    /**Kunde ''{0}'' hat noch verwaltete Konten. Kann daher nicht gelöscht werden!*/
    public static class KundeHatNochKontenExc extends multex.Exc {
        private static final long serialVersionUID = 3370553180182960011L;
    }
    

}//LgKunde

