package bank3tier.lg;

import java.sql.Date;
import java.util.List;

import org.junit.FixMethodOrder;
import org.junit.runners.MethodSorters;
import org.junit.After;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;

import bank3tier.db.DbJungeKundenQuery;
import bank3tier.db.Pool;
import bank3tier.db.Pool.ArgumentCountExc;
import bank3tier.db.Pool.ArgumentTypeExc;
import bank3tier.db.Pool.NoQueryClassExc;
import bank3tier.lg.LgKonto.BetragExc;
import bank3tier.lg.LgKonto.MindestSaldoExc;
import bank3tier.lg.LgKonto.OhneRechteExc;
import bank3tier.lg.LgKunde.KundeHatNochKontenExc;
import bank3tier.lg.LgKunde.SchonVerwalterExc;

@FixMethodOrder(MethodSorters.NAME_ASCENDING)
public class TestLgApplication extends Assert {

	//static needed here, as JUnit creates a new instance of the test driver for each test method execution.
    private static final LgSitzung _lgApplication = new LgSitzung();

    /**Erwartete Inhalte der Datenbank*/
    private static LgKunde _mueller, _gans, _sorglos;

    /**Erwartete Inhalte der Datenbank*/
    private static LgKonto _muellerGiroKonto, _muellerSparKonto, _gansKonto, _guthsmuthsVereinsKonto;


    @Before public void setUp(){
        _lgApplication.beginTransaction();
    }

    @After public void tearDown(){
        try {
            _lgApplication.endTransaction(true);
        } catch (final RuntimeException e) {
            System.err.println("Failure during " + getClass().getName() + ".tearDown():");
            multex.Msg.printReport(e);
        }
    }

    @Test public void t010KundenErfassen(){
        System.out.println("kundenErfassen:");
        //3 Musterkunden erfassen:
        _mueller = _erfassen("Manfred Mueller", "02.06.1967");
        _gans = _erfassen("Gustav Gans", "17.06.1953");
        _sorglos = _erfassen("Susi Sorglos", "09.11.1989");
        assertEquals(new Integer(2), _gans.getOid());
        assertEquals("Gustav Gans", _gans.getName());
        assertEquals("17.06.1953", _gans.getGeburt().toString());
    }

    @Test public void t020EinzelnenSuchen()
    throws Exception
    {
        System.out.println("einzelnenSuchen");
        final LgKunde sorglos
        = _lgApplication.kundenSuchen(LgDatum.fromString("09.11.1989"));
        assertEquals(_sorglos.toString(), sorglos.toString());

        final LgKunde gans = _lgApplication.getPool().find(LgKunde.class, new Integer(2));
        assertEquals(_gans.toString(), gans.toString());

        try{
            _lgApplication.kundenSuchen(LgDatum.fromString("01.01.1970"));
        	fail("Kein Kunde gefunden erwartet");
        } catch ( final RuntimeException expected){}
    }

    @Test public void t030AlleSuchen(){
        System.out.println("alleSuchen");
        final List<LgKunde> kunden = _lgApplication.alleKundenSuchen();
		assertEquals(3, kunden.size());

        final LgKunde sorglos = kunden.get(0);
        assertEquals(_sorglos.toString(), sorglos.toString());

        final LgKunde gans   = kunden.get(1);
        assertEquals(_gans.toString(), gans.toString());

        final LgKunde mueller  = kunden.get(2);
        assertEquals(_mueller.toString(), mueller.toString());

    }

    @Test public void t040Aendern()
    throws Exception
    {
        System.out.println("aendern:");

        final String gansGeburt = "17.06.1953";
		final LgKunde gansAlt = _lgApplication.kundenSuchen(LgDatum.fromString(gansGeburt));
        assertEquals(_gans.toString(), gansAlt.toString());

        gansAlt.setName("Gustl Gans");
        final String geaenderteGeburt = "31.12.2000";
		gansAlt.setGeburt(LgDatum.fromString(geaenderteGeburt));
        gansAlt.aendern();
		_gans = gansAlt;

        final LgKunde gansNeu = _lgApplication.kundenSuchen(LgDatum.fromString(geaenderteGeburt));
        assertEquals(_gans.toString(), gansNeu.toString());

        //Sollzustand wiederherstellen:
        gansNeu.setName("Gustav Gans");
        gansNeu.setGeburt(LgDatum.fromString(gansGeburt));
        gansNeu.setName("Gustav Gans");
        gansNeu.aendern();

        final LgKunde gansNeuNeu = _lgApplication.kundenSuchen(LgDatum.fromString(gansGeburt));
        assertEquals(gansNeu.toString(), gansNeuNeu.toString());
    }

    @Test public void t050JungeKundenSuchen() throws Exception {
        System.out.println("jungeKundenSuchen:");
        final List<LgKunde> kunden = _lgApplication.jungeKundenSuchen(LgDatum.fromString("01.01.1960"));
        assertEquals(2, kunden.size());

        final LgKunde mueller = kunden.get(0);
        assertEquals(_mueller.toString(), mueller.toString());

        final LgKunde sorglos  = kunden.get(1);
        assertEquals(_sorglos.toString(), sorglos.toString());
    }

    @Test public void t060KontoEroeffnen() throws Exception {
        System.out.println("kontoEroeffnen:");
        final LgKontozugriff gansVerwaltet  = _gans.kontoEroeffnen("_gansKonto");
        _gansKonto = gansVerwaltet.getKonto();
        assertTrue(gansVerwaltet.isInhaber());
        assertEquals(0, _gansKonto.getSaldo());
        assertSame(_gans, gansVerwaltet.getKunde());

        final LgKontozugriff guthsmuthsVereinsVerwaltet = _gans.kontoEroeffnen("_guthsmuthsVereinsKonto");
        assertTrue(guthsmuthsVereinsVerwaltet.isInhaber());
        _guthsmuthsVereinsKonto = guthsmuthsVereinsVerwaltet.getKonto();
        _muellerGiroKonto = _mueller.kontoEroeffnen("_muellerGiroKonto").getKonto();
        assertTrue("_muellerGiroKonto.getOid() > 0", _muellerGiroKonto.getOid() > 0);
        _muellerSparKonto = _mueller.kontoEroeffnen("_muellerSparKonto").getKonto();
        assertTrue("_muellerSparKonto.getOid() > 0", _muellerSparKonto.getOid() > 0);
        assertTrue("OIDs of _muellerGiroKonto and _muellerSparKonto must be different!", _muellerGiroKonto.getOid().intValue() != _muellerSparKonto.getOid().intValue());
    }

    /**Testet die rollenbasierte Methode LgKunde.verwalterHinzufuegen*/
    @Test public void t070VerwalterHinzufuegen() throws Exception {
        System.out.println("verwalterHinzufuegen:");
        //Sorglos als Verwalter fuer gansKonto einsetzen:
        final LgKontozugriff sorglosVerwaltet  = _gans.verwalterHinzufuegen(_gansKonto, _sorglos);
        assertFalse(sorglosVerwaltet.isInhaber());
        assertSame(_gansKonto, sorglosVerwaltet.getKonto());
        assertSame(_sorglos, sorglosVerwaltet.getKunde());

        try {
            _gans.verwalterHinzufuegen(_gansKonto, _sorglos);
            fail("SchonVerwalterExc expected");
        } catch ( final SchonVerwalterExc expected ) {
        }
        try {
            _gans.verwalterHinzufuegen(_muellerGiroKonto, _sorglos);
            fail("OhneRechteExc expected");
        } catch ( final OhneRechteExc expected ) {
        }

        //Mueller als Verwalter fuer Konto des TSV Guthsmuths einsetzen:
        _gans.verwalterHinzufuegen(_guthsmuthsVereinsKonto, _mueller);
    }

    @Test public void t080UeberweisenErfolg() throws Exception {
        System.out.println("ueberweisenErfolg:");
        final int betrag1 = 200000;
        _muellerGiroKonto.ueberweisen(_muellerSparKonto, betrag1, _mueller);
        assertEquals(betrag1, _muellerSparKonto.getSaldo());
        assertEquals(-betrag1, _muellerGiroKonto.getSaldo());
        _muellerGiroKonto.ueberweisen(_guthsmuthsVereinsKonto, 2000, _mueller);
        _gansKonto.ueberweisen(_muellerGiroKonto, 1998, _gans);
    }

    /**Testet die negativen Geschaeftsregeln beim Ueberweisen.*/
    @Test public void t090UeberweisenVerbote() throws Exception {
        System.out.println("ueberweisenVerbote:");
        try {
            _gansKonto.ueberweisen(_muellerGiroKonto, 1998, _mueller);
            fail("OhneRechteExc expected");
        } catch ( final OhneRechteExc expected ) {
        }
        try {
            _gansKonto.ueberweisen(_muellerGiroKonto, 0, _gans);
            fail("BetragExc expected");
        } catch ( final BetragExc expected ) {
        }
        try {
            _gansKonto.ueberweisen(_muellerGiroKonto, -1, _mueller);
            fail("BetragExc expected");
        } catch ( final BetragExc expected ) {
        }
        final LgKonto neuesKonto  = _gans.kontoEroeffnen("neuesKonto").getKonto();
        neuesKonto.ueberweisen(_gansKonto, -LgKonto.mindestSaldo, _gans);
        try {
            neuesKonto.ueberweisen(_muellerGiroKonto, 1, _gans);
            fail("MindestSaldoExc expected");
        } catch ( final MindestSaldoExc expected ) {
        }
    }

    @Test public void t100PrintGetInhaberKonten() throws Exception {
        System.out.println("testPrintGetInhaberKonten:");
        final List<LgKontozugriff> inhaberKontozugriffe = _lgApplication.printGetInhaberKontozugriffe();
        //Reihenfolge nach fallenden OIDs:
        _check(inhaberKontozugriffe.get(0), "Gustav Gans", "neuesKonto");
        _check(inhaberKontozugriffe.get(1), "Manfred Mueller", "_muellerSparKonto");
        _check(inhaberKontozugriffe.get(2), "Manfred Mueller", "_muellerGiroKonto");
        _check(inhaberKontozugriffe.get(3), "Gustav Gans", "_guthsmuthsVereinsKonto");
        _check(inhaberKontozugriffe.get(4), "Gustav Gans", "_gansKonto");
        assertEquals(5, inhaberKontozugriffe.size());
    }

    private void _check(final LgKontozugriff kontozugriff, final String kundeName, final String kontoBezeichnung) {
        assertTrue(kontozugriff.isInhaber());
        assertEquals(kundeName, kontozugriff.getKunde().getName());
        assertEquals(kontoBezeichnung, kontozugriff.getKonto().getBezeichnung());
    }

    @Test public void t110ReicheKundenSuchen() throws Exception {
        System.out.println("reicheKundenSuchen:");
        final int reichSaldoCent = 1000*100;
        final List<LgKontozugriff> kzs = _lgApplication.reicheKundenSuchen(reichSaldoCent);
        assertEquals(2, kzs.size());
        //Reihenfolge willkuerlich:
        final LgKunde knabe = kzs.get(0).getKunde();
        assertEquals(_gans.toString(), knabe.toString());

        final LgKunde mueller  = kzs.get(1).getKunde();
        assertEquals(_mueller.toString(), mueller.toString());
    }

    /**Tests universal pool query for exceptions
     * NoPersistentClassExc, NoQueryClassExc, ArgumentCountExc, ArgumentTypeExc
    */
    @Test public void t120QueryExceptions() throws Exception {
        System.out.println("queryExceptions:");
    	final Pool pool = _lgApplication.getPool();
        final Date date = LgDatum.fromString("01.01.1960").toDate();
        try {
            pool.findManyByQuery(LgKunde.class, LgKunde.class, new java.sql.Date[]{date});
            fail("NoQueryClassExc expected");
        } catch ( final NoQueryClassExc expected){}
        try {
            pool.findManyByQuery(LgKunde.class, DbJungeKundenQuery.class, new java.sql.Date[]{});
            fail("ArgumentCountExc expected");
        } catch ( final ArgumentCountExc expected){}
        try {
            pool.findManyByQuery(LgKunde.class, DbJungeKundenQuery.class, new java.sql.Date[]{date,date});
            fail("ArgumentCountExc expected");
        } catch ( final ArgumentCountExc expected){}
        try {
            pool.findManyByQuery(LgKunde.class, DbJungeKundenQuery.class, new Object[]{"hallo"});
            fail("ArgumentTypeExc expected");
        } catch ( final ArgumentTypeExc expected){}
    }

    @Test public void t130KundenLoeschenVerbot() throws Exception {
        System.out.println("kundenLoeschenVerbot:");
        try {
            _gans.loeschen();
            fail("KundeHatNochKontenExc expected");
        } catch (final KundeHatNochKontenExc expected) {
        }
    }
    /**Erfasst einen Kunden mit diesem Namen und diesem Geburtsdatum persistent.
     * @param i_geburt Geburtsdatum in lokaler Schreibweise
    */
    private LgKunde _erfassen(final String i_name, final String i_geburt){
        return _lgApplication.kundeErfassen(i_name, LgDatum.fromString(i_geburt));
    }


}//class
