package bank3tier.lg;

import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.junit.Assert;
import org.junit.Test;

import bank3tier.db.DbObject;
import bank3tier.db.Pool;
import bank3tier.db.DbObject.OwnerPoolChangedExc;

public class TestLgKontoMock extends Assert {
   
    private class DummyPool implements Pool {

        //Dummy implementations for all methods of Pool:
        public void insert(DbObject io_object) throws OwnerPoolChangedExc { //dummy
        }
        public void reattach(DbObject io_object) { //dummy
        }
        public boolean save(DbObject io_object) throws ConcurrentUpdateExc,
                OwnerPoolChangedExc {return false;}
        public boolean delete(DbObject io_object) {
            return false;
        }
        public boolean areLinked(DbObject i_left, Class<? extends DbObject> i_linkClass, DbObject i_right) throws NoPersistentClassExc {
            return false;
        }
        public DbObject getLink(DbObject i_object, Class i_partnerClass)
                throws LinkFailure, NoPersistentClassExc {
            return null;
        }
        public <T extends DbObject> boolean hasLinks(Class<T> i_resultClass, DbObject i_object) throws LinkFailure, NoPersistentClassExc {
            return false;
        }
        public <T extends DbObject> List<T> getLinks(Class<T> i_resultClass, DbObject i_object) throws LinkFailure, NoPersistentClassExc{
            return null;
        }
        public <T extends DbObject> T find(Class<T> i_persistentClass, Integer i_oid) throws NoPersistentClassExc, OidNotFoundExc {
            return null;
        }
        public <T extends DbObject> List<T> findAll(Class<T> i_persistentClass) throws NoPersistentClassExc {
            return null;
        }
        public <T extends DbObject> List<T> findManyByQuery(Class<T> i_resultClass, Class i_queryClass, Object[] i_args) throws NoPersistentClassExc, NoQueryClassExc, ArgumentCountExc, ArgumentTypeExc {
            return null;
        }
        public <T extends DbObject> T findOneBykey(Class<T> i_persistentClass, String i_keyFieldName, Object i_keyFieldValue) throws NoPersistentClassExc {
            return null;
        }
        public <T extends DbObject> List<T> findManyByKey(Class<T> i_persistentClass, String i_keyFieldName, Object i_keyFieldValue) {
            return null;
        }
    }
    
    @Test public void ueberweisen_OhneRechteExc() throws Exception {
        final Pool pool = new DummyPool();
        final LgKonto gansKonto = new LgKonto(pool);
        final LgKunde knabe = new LgKunde();
        final LgKonto muellerKonto = new LgKonto(pool);
        try{
            gansKonto.ueberweisen(muellerKonto, 12345, knabe);
            fail("LgKonto.OhneRechteExc expected");
        } catch ( LgKonto.OhneRechteExc expected){
        }
    }
    
    /**Eine Mock-Implementierung arbeitet aktiv am Testen mit.*/
    private class UeberweisenMockPool extends DummyPool {
        private final Set<DbObject> _saved = new HashSet<DbObject>();

        @Override
        public boolean areLinked(
            DbObject i_left, Class i_linkClass, DbObject i_right
        ){
            return true;
        }
        
        @Override
        public void reattach(DbObject io_object) 
        {
            _saved.add(io_object);
        }
        
        public void checkSaved(final LgKonto i_gansKonto, final LgKonto i_muellerKonto) throws Exception {
            assertEquals(2, _saved.size());
            assertNotSame(i_gansKonto, i_muellerKonto);
            assertTrue(_saved.contains(i_gansKonto));
            assertTrue(_saved.contains(i_muellerKonto));
        }
    }
    
    @Test public void ueberweisen() throws Exception {
        final UeberweisenMockPool pool = new UeberweisenMockPool();
        final LgKunde knabe = new LgKunde();
        final LgKonto gansKonto = new LgKonto(pool);
        final LgKonto muellerKonto = new LgKonto(pool);

        gansKonto.ueberweisen(muellerKonto, -LgKonto.mindestSaldo, knabe);

        assertEquals(+LgKonto.mindestSaldo, gansKonto.getSaldo());
        assertEquals(-LgKonto.mindestSaldo, muellerKonto.getSaldo());
        pool.checkSaved(gansKonto, muellerKonto);
    }
    
    @Test public void ueberweisen_MindestSaldoExc() throws Exception {
        final Pool pool = new DummyPool(){
            @Override
            public boolean areLinked(
                DbObject i_left, Class i_linkClass, DbObject i_right
            ){   return true;   }
        };
        final LgKonto gansKonto = new LgKonto(pool);
        final LgKunde knabe = new LgKunde();
        final LgKonto muellerKonto = new LgKonto(pool);
        try{
            gansKonto.ueberweisen(muellerKonto, -LgKonto.mindestSaldo + 1, knabe);
            fail("LgKonto.MindestSaldoExc expected");
        } catch ( LgKonto.MindestSaldoExc expected){
        }
    }


}//class
